/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.dbcopy;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import com.intland.codebeamer.dbcopy.JDBCUtils.IResultSetHandler;

/**
 * Handler for fetching the rows one-by-one workaround.
 * This handler receives the ids of the records only and executes 1 other select per id to fetch the full row.
 *
 * @author <a href="mailto:zoltan.luspai@intland.com">Zoltan Luspai</a>
 * @version $Id: DatabaseCopier.java 18052 2007-03-26 16:11:27Z zluspai $
 */
class FetchOneByOneHandler implements JDBCUtils.IResultSetHandler {
	private final PreparedStatement fetchOneRecordStatement;
	private IResultSetHandler delegate;

	/**
	 * Constructor
	 * @param fetchOneRecordStatement
	 * @param delegate The real handler for the result row when it is fetched
	 */
	FetchOneByOneHandler(PreparedStatement fetchOneRecordStatement, IResultSetHandler delegate) {
		this.fetchOneRecordStatement = fetchOneRecordStatement;
		this.delegate = delegate;
	}

	public Object handleResults(ResultSet rs) throws SQLException {
		ResultSet oneRecord = null;
		try {
			Object key = rs.getObject(1);
			fetchOneRecordStatement.clearParameters();
			fetchOneRecordStatement.setObject(1, key);
			fetchOneRecordStatement.execute();
			oneRecord = fetchOneRecordStatement.getResultSet();
			oneRecord.next();
			return delegate.handleResults(oneRecord);
		} finally {
			if (oneRecord != null) {
				oneRecord.close();
			}
		}
	}
}
