/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.dbcopy;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import sun.misc.BASE64Decoder;

/**
 * Class will fix the difference in the password crypting functions between databases.
 *
 * @author <a href="mailto:zoltan.luspai@intland.com">Zoltan Luspai</a>
 * @version $Id$
 */
public class PasswordFixer {
	private final static Logger logger = Logger.getLogger(PasswordFixer.class);

	// the name of the users table
	protected String users_table_name = "users";

	/**
	 * Fix the passwords when migrating.
	 * @param conn
	 * @param databaseVersions
	 * @throws SQLException
	 * @return The summary result string about the conversion
	 */
	public String fixPasswords(final Connection conn, String databaseVersions[]) throws SQLException {
		// only works with database versions <=4.2.3
		String version = databaseVersions[0] + databaseVersions[1];
		if (!version.startsWith("4.2")) {
			String msg = "Only passwords in databases version 4.2.x can be converted!";
			logger.warn(msg);
			return msg;
		}
		conn.setAutoCommit(false);

		String result = (String) JDBCUtils.query(conn, "select id, unused1, name from " + users_table_name, new JDBCUtils.IResultSetHandler(){

			public Object handleResults(ResultSet rs) throws SQLException {
				// names of users whose passwords could not be converted
				StringBuffer failedUsers = new StringBuffer();
				String dbPasswordFunction = getDbPasswordFunction(conn);

				String updatePasswdSQL = "UPDATE " + users_table_name + " SET passwd = " + dbPasswordFunction + "( ? ) WHERE id= ?";
				PreparedStatement updateStmt = conn.prepareStatement(updatePasswdSQL);

				while (rs.next()) {
					String id = rs.getString(1);
					String unused1 = rs.getString(2);
					String name = rs.getString(3);
					logger.debug("Fixing password for: id=" + id + ",key=" + unused1);

					try {
						// decode base64 password
						BASE64Decoder decoder = new BASE64Decoder();
						String passwd = null;
						if (unused1 != null) {
							try {
								passwd = new String(decoder.decodeBuffer(unused1), "UTF-8");
							} catch (Exception e) {
								throw new RuntimeException(e);
							}

							// update password
							updateStmt.clearParameters();
							updateStmt.setString(1, passwd);
							updateStmt.setString(2, id);
							int numUpdated = updateStmt.executeUpdate();
							conn.commit();
							if (numUpdated != 1) {
								throw new RuntimeException("Unexpected number of updates, only one row should be updated, but " + numUpdated +" werw, id=" + id);
							}
						}
					} catch (Exception ex) {
						logger.warn("Could not migrate password for user id:" + id + ", name:" + name +", (reason: " + ex.getMessage()+")", ex);
						if (failedUsers.length() !=0) {
							failedUsers.append(",");
						}
						failedUsers.append(name);
					}
				}

				updateStmt.close();

				if (failedUsers.length() == 0) {
					return "OK";
				}
				return "WARNING: passwords of users '" + failedUsers +"' are not converted, please review log for details!";
			}

		});
		return result;
	}

	// get the database specific password function
	protected String getDbPasswordFunction(Connection conn1) throws SQLException {
		String dbName = JDBCUtils.getDBName(conn1);
		if (dbName.equalsIgnoreCase(JDBCUtils.DBNAME_MY_SQL)) {
			return "MD5";
		} else if (dbName.equalsIgnoreCase(JDBCUtils.DBNAME_ORACLE)) {
			return "PASSWORD_ENCRYPT";
		} else if (dbName.equalsIgnoreCase(JDBCUtils.DBNAME_POSTGRE_SQL)) {
			return "md5";
		} else if (dbName.equalsIgnoreCase(JDBCUtils.DBNAME_DERBY)) {
			return "OLD_PASSWORD";
		}
		return "";
	}

}
